==> dna-0.2.0.tar.gz: example/opengl_test.dart <==
import 'dart:math';

//import 'package:dartgl/dartgl.dart';
import 'package:dartgl/dartgl_sdl.dart';
import 'opengl.dart';
import 'dart:typed_data';
import 'dart:convert';

var gl;

var arrays;
var textures;
var program;
var location;
const int COUNT = 10;

void main() {

  SDL_Init(SDL_INIT_VIDEO);
  var window = SDL_CreateWindow("Dart+OpenGL", 100, 100, 600, 400,
      SDL_WINDOW_OPENGL|SDL_WINDOW_RESIZABLE);
  var context = SDL_GL_CreateContext(window);

  SDL_GL_MakeCurrent(window, context);
  SDL_GL_SetSwapInterval(1);

  prepare();

  double time = 0.0;
  int count = 0;
  var exit = false;

  var startTime = new DateTime.now();
  var event = new SDL_Event();
  while (!exit){

    while (SDL_PollEvent(event) != 0){
      if (event.type == 0x100) {
        exit = true;
      }
    }

    time += PI/64;
    count++;
    draw(time);

    SDL_GL_SwapWindow(window);
  }
  //print(count/new DateTime.now().difference(startTime).inSeconds);

  SDL_GL_DeleteContext(context);
  SDL_DestroyWindow(window);

  SDL_Quit();
}


void draw(double time){
  gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  gl.glUseProgram(program);
  gl.glUniform1f(location, sin(time)*0.2 + 0.8);

  gl.glActiveTexture(GL_TEXTURE0);
  gl.glBindTexture(GL_TEXTURE_2D, textures[0]);
  gl.glUniform1i(gl.glGetUniformLocation(program, "texture1"), 0);

  gl.glBindVertexArray(arrays[0]);          // select first VAO
  for (int i = 0; i < 100; i++){
    gl.glDrawArrays(GL_TRIANGLES, 0, 6);      // draw first object
  }

  gl.glBindVertexArray(0);
}

void prepare(){

  gl = new OpenGL();

  gl.glClearColor(0.0, 0.7, 0.99, 0.0);

  var vertexShader = gl.glCreateShader(GL_VERTEX_SHADER);
  shader(vertexShader, v);

  var fragmentShader = gl.glCreateShader(GL_FRAGMENT_SHADER);
  shader(fragmentShader, f);

  program = gl.glCreateProgram();
  gl.glAttachShader(program, vertexShader);
  gl.glAttachShader(program, fragmentShader);

  gl.glBindAttribLocation(program, 0, "in_Position");
  gl.glBindAttribLocation(program, 1, "in_Color");
  gl.glBindAttribLocation(program, 2, "in_Texture");

  gl.glLinkProgram(program);
  location = gl.glGetUniformLocation(program, "in_Time");

  arrays = new Uint32List(1);
  gl.glGenVertexArrays(1, arrays);



  var data = new Float32List.fromList([1.0, 1.0, -1.0, -1.0, -1.0,-1.0, 1.0, -1.0, -1.0,
  1.0, 1.0, -1.0, -1.0, -1.0,-1.0, -1.0, 1.0, -1.0]);
  var data2 = new Float32List.fromList([1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0,
  1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0]);
  var data3 = new Float32List.fromList([0.0, 0.0, 1.0, 1.0, 0.0, 1.0,
  0.0, 0.0, 1.0, 1.0, 1.0, 0.0]);

  var buffers = new Uint32List(3);
  gl.glGenBuffers(3, buffers);

  gl.glBindVertexArray(arrays[0]);

  gl.glBindBuffer(GL_ARRAY_BUFFER, buffers[0]);
  gl.glBufferData(GL_ARRAY_BUFFER, data.lengthInBytes, data, GL_STATIC_DRAW);

  gl.glVertexAttribPointer(0, 3, GL_FLOAT, false, 0, 0);
  gl.glEnableVertexAttribArray(0);

  gl.glBindBuffer(GL_ARRAY_BUFFER, buffers[1]);
  gl.glBufferData(GL_ARRAY_BUFFER, data2.lengthInBytes, data2, GL_STATIC_DRAW);
  gl.glVertexAttribPointer(1, 3, GL_FLOAT, false, 0, 0);
  gl.glEnableVertexAttribArray(1);

  gl.glBindBuffer(GL_ARRAY_BUFFER, buffers[2]);
  gl.glBufferData(GL_ARRAY_BUFFER, data3.lengthInBytes, data3, GL_STATIC_DRAW);
  gl.glVertexAttribPointer(2, 2, GL_FLOAT, false, 0, 0);
  gl.glEnableVertexAttribArray(2);

  textures = new Uint32List(1);
  gl.glGenTextures(1, textures);

  gl.glBindTexture(GL_TEXTURE_2D, textures[0]);

  gl.glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 2, 0, GL_RGBA, GL_UNSIGNED_BYTE, new Uint32List.fromList([0,1000000,0,1000000]));

}

void shader(int shader, String text) {

  var lines = text.split("\n").map((l)=>l+"\n").toList();
  var lengths =  lines.map((l)=>l.length).toList();

  gl.glShaderSource(shader, lines.length, lines, null);

  gl.glCompileShader(shader);

  var status = new Int32List(1);
  gl.glGetShaderiv(shader, GL_COMPILE_STATUS, status);
  if (status[0] != GL_TRUE){
    print("shader status ${status[0]}");

    Int64List length = new Int64List(1);
    TypedData infoLog = new ByteData(256+1);
    gl.glGetShaderInfoLog(shader, 256, length, infoLog);

    if (length[0] > 0){
      print(UTF8.decode(new Uint8List.view(infoLog.buffer, 0, length[0])..toList()));
    }
  }
}

void glInfo() {
  print("GL_VENDOR: ${gl.glGetString(GL_VENDOR)}");
  print("GL_RENDERER: ${gl.glGetString(GL_RENDERER)}");
  print("GL_VERSION: ${gl.glGetString(GL_VERSION)}");
}

var v = """
#version 130

uniform float in_Time;
in vec3 in_Position;
in vec3 in_Color;
in vec2 in_Texture;
out vec3 ex_Color;
out vec2 ex_Texture;

void main(void)
{
  gl_Position = vec4(in_Position.x*in_Time, in_Position.y*in_Time, in_Position.z*in_Time, 1.0);
  ex_Color = vec3(in_Color.r, in_Color.g, in_Color.b);
  ex_Texture = in_Texture;
}
""";

var f = """
#version 130

precision highp float; // needed only for version 1.30

uniform sampler2D texture1;
in vec3 ex_Color;
in vec2 ex_Texture;
out vec4 out_Color;

void main(void)
{
  //out_Color = texture(texture1, ex_Texture).xyzw;
  out_Color = vec4(ex_Color,1.0);
}
""";

==> dna-0.2.1.tar.gz: example/opengl_test.dart <==
import 'dart:math';

import '../lib/dna.dart';
import 'opengl.dart';
import 'sdl.dart';
import 'dart:typed_data';
import 'dart:convert';

var gl;
var sdl;

var arrays;
var textures;
var program;
var location;
const int COUNT = 10;

void main() {

  var sdl = new Sdl();

  sdl.SDL_Init(SDL_INIT_VIDEO);
  var window = sdl.SDL_CreateWindow("Dart+OpenGL", 100, 100, 600, 400,
      SDL_WINDOW_OPENGL|SDL_WINDOW_RESIZABLE);
  var context = sdl.SDL_GL_CreateContext(window);

  sdl.SDL_GL_MakeCurrent(window, context);
  sdl.SDL_GL_SetSwapInterval(0);

  prepare();

  double time = 0.0;
  int count = 0;
  var exit = false;

  var startTime = new DateTime.now();
  var event = new Ref(new SDL_Event());
  while (!exit){

    while (sdl.SDL_PollEvent(event) != 0){
      if (event.value.type == 0x100) {
        exit = true;
      }
    }

    time += PI/64;
    count++;
    draw(time);

    sdl.SDL_GL_SwapWindow(window);
  }
  print(count/new DateTime.now().difference(startTime).inSeconds);

  sdl.SDL_GL_MakeCurrent(window, 0);
  sdl.SDL_GL_DeleteContext(context);
  sdl.SDL_DestroyWindow(window);

  sdl.SDL_Quit();
}


void draw(double time){
  gl.glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  gl.glUseProgram(program);
  gl.glUniform1f(location, sin(time)*0.2 + 0.8);

  gl.glActiveTexture(GL_TEXTURE0);
  gl.glBindTexture(GL_TEXTURE_2D, textures[0]);
  gl.glUniform1i(gl.glGetUniformLocation(program, "texture1"), 0);

  gl.glBindVertexArray(arrays[0]);          // select first VAO
  for (int i = 0; i < 100; i++){
    gl.glDrawArrays(GL_TRIANGLES, 0, 6);      // draw first object
  }

  gl.glBindVertexArray(0);
}

void prepare(){

  gl = new OpenGL();

  gl.glClearColor(0.0, 0.7, 0.99, 0.0);

  var vertexShader = gl.glCreateShader(GL_VERTEX_SHADER);
  shader(vertexShader, v);

  var fragmentShader = gl.glCreateShader(GL_FRAGMENT_SHADER);
  shader(fragmentShader, f);

  program = gl.glCreateProgram();
  gl.glAttachShader(program, vertexShader);
  gl.glAttachShader(program, fragmentShader);

  gl.glBindAttribLocation(program, 0, "in_Position");
  gl.glBindAttribLocation(program, 1, "in_Color");
  gl.glBindAttribLocation(program, 2, "in_Texture");

  gl.glLinkProgram(program);
  location = gl.glGetUniformLocation(program, "in_Time");

  arrays = new Uint32List(1);
  gl.glGenVertexArrays(1, arrays);



  var data = new Float32List.fromList([1.0, 1.0, -1.0, -1.0, -1.0,-1.0, 1.0, -1.0, -1.0,
  1.0, 1.0, -1.0, -1.0, -1.0,-1.0, -1.0, 1.0, -1.0]);
  var data2 = new Float32List.fromList([1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0,
  1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0]);
  var data3 = new Float32List.fromList([0.0, 0.0, 1.0, 1.0, 0.0, 1.0,
  0.0, 0.0, 1.0, 1.0, 1.0, 0.0]);

  var buffers = new Uint32List(3);
  gl.glGenBuffers(3, buffers);

  gl.glBindVertexArray(arrays[0]);

  gl.glBindBuffer(GL_ARRAY_BUFFER, buffers[0]);
  gl.glBufferData(GL_ARRAY_BUFFER, data.lengthInBytes, data, GL_STATIC_DRAW);

  gl.glVertexAttribPointer(0, 3, GL_FLOAT, false, 0, 0);
  gl.glEnableVertexAttribArray(0);

  gl.glBindBuffer(GL_ARRAY_BUFFER, buffers[1]);
  gl.glBufferData(GL_ARRAY_BUFFER, data2.lengthInBytes, data2, GL_STATIC_DRAW);
  gl.glVertexAttribPointer(1, 3, GL_FLOAT, false, 0, 0);
  gl.glEnableVertexAttribArray(1);

  gl.glBindBuffer(GL_ARRAY_BUFFER, buffers[2]);
  gl.glBufferData(GL_ARRAY_BUFFER, data3.lengthInBytes, data3, GL_STATIC_DRAW);
  gl.glVertexAttribPointer(2, 2, GL_FLOAT, false, 0, 0);
  gl.glEnableVertexAttribArray(2);

  textures = new Uint32List(1);
  gl.glGenTextures(1, textures);

  gl.glBindTexture(GL_TEXTURE_2D, textures[0]);

  gl.glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 2, 0, GL_RGBA, GL_UNSIGNED_BYTE, new Uint32List.fromList([0,1000000,0,1000000]));

}

void shader(int shader, String text) {

  var lines = text.split("\n").map((l)=>l+"\n").toList();
  var lengths =  lines.map((l)=>l.length).toList();

  gl.glShaderSource(shader, lines.length, lines, null);

  gl.glCompileShader(shader);

  var status = new Int32List(1);
  gl.glGetShaderiv(shader, GL_COMPILE_STATUS, status);
  if (status[0] != GL_TRUE){
    print("shader status ${status[0]}");

    Int64List length = new Int64List(1);
    TypedData infoLog = new ByteData(256+1);
    gl.glGetShaderInfoLog(shader, 256, length, infoLog);

    if (length[0] > 0){
      print(UTF8.decode(new Uint8List.view(infoLog.buffer, 0, length[0])..toList()));
    }
  }
}

void glInfo() {
  print("GL_VENDOR: ${gl.glGetString(GL_VENDOR)}");
  print("GL_RENDERER: ${gl.glGetString(GL_RENDERER)}");
  print("GL_VERSION: ${gl.glGetString(GL_VERSION)}");
}

var v = """
#version 130

uniform float in_Time;
in vec3 in_Position;
in vec3 in_Color;
in vec2 in_Texture;
out vec3 ex_Color;
out vec2 ex_Texture;

void main(void)
{
  gl_Position = vec4(in_Position.x*in_Time, in_Position.y*in_Time, in_Position.z*in_Time, 1.0);
  ex_Color = vec3(in_Color.r, in_Color.g, in_Color.b);
  ex_Texture = in_Texture;
}
""";

var f = """
#version 130

precision highp float; // needed only for version 1.30

uniform sampler2D texture1;
in vec3 ex_Color;
in vec2 ex_Texture;
out vec4 out_Color;

void main(void)
{
  //out_Color = texture(texture1, ex_Texture).xyzw;
  out_Color = vec4(ex_Color,1.0);
}
""";
